# -*- coding: utf-8 -*-

#version de Python
import sys
print(sys.version)

#changement de dossier
import os
os.chdir("D:/DataMining/Databases_for_mining/dataset_for_soft_dev_and_comparison/mlxtend_python")

#train
import pandas
DTrain = pandas.read_excel("spam_mlxtend.xlsx",sheet_name="spam_train")
print(DTrain.info())

#test
DTest = pandas.read_excel("spam_mlxtend.xlsx",sheet_name="spam_test")
print(DTest.shape)

#transformation échantillon apprentissage
from sklearn.preprocessing import StandardScaler
stds = StandardScaler()
ZTrain = stds.fit_transform(X=DTrain.iloc[:,1:])
print(ZTrain.mean(axis=0))

#et de l'échantillon test
ZTest = stds.transform(DTest.iloc[:,1:])
print(ZTest.mean(axis=0))

#isoler les cibles par commodité
YTrain = DTrain.iloc[:,0]
YTest = DTest.iloc[:,0]

#distribution de la cible en test
print(YTest.value_counts())

#modélisation avec un SVM Rbf
from sklearn.svm import SVC
m_svm = SVC(gamma='scale',probability=True)
m_svm.fit(ZTrain,YTrain)

#prédiction en test
p_svm = m_svm.predict(ZTest)

#distribution
import numpy
print(numpy.unique(p_svm,return_counts=True))

#version de mlxtend
import mlxtend
print(mlxtend.__version__)

#matrice de confusion
from mlxtend.evaluate import confusion_matrix
cm_svm = confusion_matrix(YTest,p_svm)
print(cm_svm)

#plotter la matrice de confusion
from mlxtend.plotting import plot_confusion_matrix
plot_confusion_matrix(conf_mat=cm_svm)

#outil estimation des différents indicateurs
from mlxtend.evaluate import scoring

#taux de reconnaissance
print(scoring(YTest,p_svm,metric='accuracy'))

#taux d'erreur
print(scoring(YTest,p_svm,metric='error'))

#rappel
print(scoring(YTest,p_svm,metric='recall',positive_label="yes"))

#precision
print(scoring(YTest,p_svm,metric='precision',positive_label="yes"))

#F1-Score
print(scoring(YTest,p_svm,metric='f1',positive_label="yes"))

#estimation 0.632 bootstrap de l'erreur
import numpy
from mlxtend.evaluate import bootstrap_point632_score
accuracies = bootstrap_point632_score(m_svm,ZTrain,YTrain,method='.632',n_splits=200)

#on dispose de 200 valeurs de l'accuracy
print(accuracies.shape)

#histogramme
import matplotlib.pyplot as plt
plt.hist(accuracies)

#mediane des valeurs
print(numpy.median(accuracies))

#intervalle de confiance empirique
print('90%% CI : [%.2f,%.2f]' % (100*numpy.percentile(accuracies,5.0),100*numpy.percentile(accuracies,95.0)))

#un second modèle : régression logistique
from sklearn.linear_model import LogisticRegression
m_lr = LogisticRegression(solver='lbfgs')
m_lr.fit(ZTrain,YTrain)

#son taux de reconnaissance
p_lr = m_lr.predict(ZTest)
print(scoring(YTest,p_lr,metric='accuracy'))

#comparer les performances - Test de McNemar
#tableau de comparaison
from mlxtend.evaluate import mcnemar_table
tb = mcnemar_table(y_target=YTest,y_model1=p_svm,y_model2=p_lr)
print(tb)

#statistique de test et conclusion
from mlxtend.evaluate import mcnemar
chi2, pval_mcnemar = mcnemar(ary=tb, corrected=False)
print('chi-squared %.5f' % chi2)
print('p-value: %.5f' % pval_mcnemar)

##généralisation avec le test de Cochran
#from mlxtend.evaluate import cochrans_q
#q, pval_cochran = cochrans_q(numpy.asarray(YTest),p_svm,p_lr)
#print('Q: %.5f' % q)
#print('p-value: %.5f' % pval_cochran)

#comparaison de performances par resampling
from mlxtend.evaluate import paired_ttest_kfold_cv
tcv, pval_cv = paired_ttest_kfold_cv(estimator1=m_svm,estimator2=m_lr,cv=30,scoring='accuracy',X=ZTrain,y=YTrain,random_seed=1)
print('t: %.5f' % tcv)
print('p-value: %.5f' % pval_cv)

#combinaison de classifieurs - Vote simple
from mlxtend.classifier import EnsembleVoteClassifier
m_ev = EnsembleVoteClassifier(clfs=[m_svm,m_lr], weights=[1.0,1.0], voting='soft')

#nécessité de ré-apprendre
m_ev.fit(ZTrain,YTrain)

#prédiction
p_ev = m_ev.predict(ZTest)

#performances
print(scoring(YTest,p_ev,metric='accuracy'))

#définir un meta-classifieur
meta_clf = LogisticRegression(solver='lbfgs')

#combinaison de classifieurs - Stacking
from mlxtend.classifier import StackingCVClassifier
m_stacking = StackingCVClassifier(classifiers=[m_svm,m_lr],use_probas=True,meta_classifier=meta_clf,cv=5)

#apprentissage
m_stacking.fit(ZTrain,YTrain)

#prédiction
p_stacking = m_stacking.predict(ZTest)

#performances
print(scoring(YTest,p_stacking,metric='accuracy'))

#comparer les 4 modèles - Test de Cochran
from mlxtend.evaluate import cochrans_q
q, pval_cochran = cochrans_q(numpy.asarray(YTest),p_svm,p_lr,p_ev,p_stacking)
print('Q: %.5f' % q)
print('p-value: %.5f' % pval_cochran)

#comparons le plus faible et le plus fort - sorte de test post hoc
#table
tbb = mcnemar_table(y_target=YTest,y_model1=p_lr,y_model2=p_stacking)
print(tbb)

#test stat
chi2b, pval_mcnemarb = mcnemar(ary=tbb, corrected=False)
print('chi-squared %.5f' % chi2b)
print('p-value: %.5f' % pval_mcnemarb)

#importance des variables dans le stacking
from mlxtend.evaluate import feature_importance_permutation
fimp_mean,fimp_all = feature_importance_permutation(predict_method=m_stacking.predict,X=ZTest,y=YTest,metric='accuracy',num_rounds=20,seed=1)

#valeurs moyennes
print(fimp_mean)

#index des valeurs triées de manière décroissante
index = numpy.argsort(fimp_mean)[::-1]

#tableau importance des variables
fimp = pandas.DataFrame({'variable':DTrain.columns[1:][index],'importance':fimp_mean[index]})

#affichage des 10 premiers
print(fimp.head(10))

#affichage graphique
plt.barh(DTrain.columns[1:][index][:10][::-1],fimp_mean[index][:10][::-1])

#représentation des points dans l'espace des 2 variables les plus discriminantes
from mlxtend.plotting import category_scatter
category_scatter(x='wf_hp', y='wf_remove', label_col='spam', data=DTrain, legend_loc='upper left',markersize=20,alpha=0.3)

#selection de variables - approche wrapper sur validation croisée - 20 variables max
from mlxtend.feature_selection import SequentialFeatureSelector as SFS
sfs = SFS(m_lr,k_features=20,forward=True,scoring='accuracy',cv=5,verbose=True)

#recherche des meilleures variables
sfs.fit(ZTrain,numpy.asarray(YTrain=='yes',dtype='int'),custom_feature_names=DTrain.columns[1:])

#1ere etape
sfs.subsets_[1]

#20e étape
sfs.subsets_[20]

#affichage graphique
from mlxtend.plotting import plot_sequential_feature_selection as plot_sfs
plot_sfs(sfs.get_metric_dict(), kind='std_dev')

#etape 11
sfs.subsets_[11]

#récupération des numéros des variables concernées
num_sel_var = list(sfs.subsets_[11]['feature_idx'])
print(num_sel_var)

#restriction du matrices de données
ZTrainSel = ZTrain[:,num_sel_var]
ZTestSel = ZTest[:,num_sel_var]

#apprentissage
m_svmSel = SVC(gamma='scale',probability=True)
m_svmSel.fit(ZTrainSel,YTrain)

#prédiction en test
p_svmSel = m_svmSel.predict(ZTestSel)

#taux de reconnaissance
print(scoring(YTest,p_svmSel,metric='accuracy'))

